@@ -82,6 +82,8 @@ def connected(func):
                 self.show_output("Authentication failed.")
             except NoAuthError:
                 self.show_output("Not authenticated.")
+            except BadVersionError:
+                self.show_output("Bad version.")
             except ConnectionLoss:
                 self.show_output("Connection loss.")
             except NotReadOnlyCallError:
@@ -95,18 +97,19 @@ def connected(func):
 
 
 def check_path_exists_foreach(paths, func):
+    """ check that paths exist (unless we are in a transaction) """
     @wraps(func)
     def wrapper(*args):
         self = args[0]
         params = args[1]
 
-        for path_param_name in paths:
-            path = getattr(params, path_param_name)
-            path = self.resolve_path(path)
-            setattr(params, path_param_name, path)
-            if not self.client.exists(path):
-                self.show_output("Path %s=%s doesn't exist", path_param_name, path)
-                return False
+        if not self.in_transaction:
+            for param_name in paths:
+                path = self.resolve_path(getattr(params, param_name))
+                setattr(params, param_name, path)
+                if not self.client.exists(path):
+                    self.show_output("Path %s=%s doesn't exist", param_name, path)
+                    return False
 
         return func(self, params)
 
@@ -119,16 +122,27 @@ def check_paths_exists(*paths):
 
 
 def check_path_absent(func):
-    """ check path doesn't exist """
+    """
+    check path doesn't exist (unless we are in a txn or it's sequential)
+
+    note: when creating sequential znodes, a trailing slash means no prefix, i.e.:
+
+        create(/some/path/, sequence=True) -> /some/path/0000001
+
+    for all other cases, it's dropped.
+    """
     @wraps(func)
     def wrapper(*args):
         self = args[0]
         params = args[1]
-        path = params.path
-        params.path = self.resolve_path(path)
-        if not self.client.exists(params.path):
+        orig_path = params.path
+        sequence = getattr(params, 'sequence', False)
+        params.path = self.resolve_path(params.path)
+        if self.in_transaction or sequence or not self.client.exists(params.path):
+            if sequence and orig_path.endswith("/") and params.path != "/":
+                params.path += "/"
             return func(self, params)
-        self.show_output("Path %s already exists", path)
+        self.show_output("Path %s already exists", params.path)
     return wrapper
 
 
@@ -170,6 +184,7 @@ class Shell(XCmd):
         self._read_only = read_only
         self._async = async
         self._zk = None
+        self._txn = None        # holds the current transaction, if any
         self.connected = False
         self.state_transitions_enabled = True
 
@@ -413,6 +428,7 @@ class Shell(XCmd):
            /some/path (in the connected server)
            zk://[scheme:user:passwd@]host/<path>
            json://!some!path!backup.json/some/path
+           file:///some/file
 
         with a few restrictions. Given the semantic differences that znodes have with filesystem
         directories recursive copying from znodes to an fs could lose data, but to a JSON file it
@@ -422,7 +438,8 @@ class Shell(XCmd):
 
         > cp /some/znode /backup/copy-znode  # local
         > cp /some/znode zk://digest:bernie:pasta@10.0.0.1/backup true true
-        > cp /some/path json://!home!user!backup.json/ true true
+        > cp /some/znode json://!home!user!backup.json/ true true
+        > cp file:///tmp/file /some/zone  # fs to zk
 
         """
         try:
@@ -735,6 +752,7 @@ class Shell(XCmd):
             start = len(results) + params.top if abs(params.top) < len(results) else 0
             end = len(results)
 
+        offs = 1 if params.path == "/" else len(params.path) + 1
         for i in range(start, end):
             path, stat = results[i]
 
@@ -743,7 +761,7 @@ class Shell(XCmd):
                 time.ctime(stat.created).ljust(32),
                 time.ctime(stat.last_modified).ljust(32),
                 ("0x%x" % stat.ephemeralOwner).ljust(23),
-                path[len(params.path) + 1:]
+                path[offs:]
             )
 
     def complete_summary(self, cmd_param_text, full_cmd, *rest):
@@ -999,18 +1017,14 @@ class Shell(XCmd):
 
         """
         try:
-            self._zk.create(params.path,
-                            decoded(params.value),
-                            acl=None,
-                            ephemeral=params.ephemeral,
-                            sequence=params.sequence,
-                            makepath=params.recursive)
+            kwargs = {"acl": None, "ephemeral": params.ephemeral, "sequence": params.sequence}
+            if not self.in_transaction:
+                kwargs["makepath"] = params.recursive
+            self.client_context.create(params.path, decoded(params.value), **kwargs)
         except NodeExistsError:
             self.show_output("Path %s exists", params.path)
         except NoNodeError:
-            self.show_output(
-                "Part of the parent path for %s doesn't exist (try recursive)",
-                params.path)
+            self.show_output("Missing path in %s (try recursive?)", params.path)
 
     def complete_create(self, cmd_param_text, full_cmd, *rest):
         complete_value = partial(complete_values, ["somevalue"])
@@ -1024,47 +1038,175 @@ class Shell(XCmd):
         return complete(completers, cmd_param_text, full_cmd, *rest)
 
     @connected
-    @ensure_params(Required("path"), Required("value"))
+    @ensure_params(Required("path"), Required("value"), IntegerOptional("version", -1))
     @check_paths_exists("path")
     def do_set(self, params):
         """
         Updates the znode's value
 
-        set <path> <value>
+        set <path> <value> [version]
 
-        Example:
+        Examples:
 
         > set /foo 'bar'
+        > set /foo 'verybar' 3
 
         """
-        self._zk.set(params.path, decoded(params.value))
+        self.set(params.path, decoded(params.value), version=params.version)
 
     def complete_set(self, cmd_param_text, full_cmd, *rest):
-        """ TODO: suggest the old value """
+        """ TODO: suggest the old value & the current version """
         complete_value = partial(complete_values, ["updated-value"])
-        completers = [self._complete_path, complete_value]
+        complete_version = partial(complete_values, [str(i) for i in range(1, 11)])
+        completers = [self._complete_path, complete_value, complete_version]
         return complete(completers, cmd_param_text, full_cmd, *rest)
 
     @connected
-    @ensure_params(Required("path"))
+    @ensure_params(Required("path"), IntegerOptional("version", -1))
     @check_paths_exists("path")
+    def do_zero(self, params):
+        """
+        Set the znode's to None (no bytes)
+
+        zero <path> [version]
+
+        Examples:
+
+        > zero /foo
+        > zero /foo 3
+
+        """
+        self.set(params.path, None, version=params.version)
+
+    def complete_zero(self, cmd_param_text, full_cmd, *rest):
+        """ TODO: suggest the current version """
+        complete_version = partial(complete_values, [str(i) for i in range(1, 11)])
+        completers = [self._complete_path, complete_version]
+        return complete(completers, cmd_param_text, full_cmd, *rest)
+
+    def set(self, path, value, version):
+        """ sets a znode's data """
+        if self.in_transaction:
+            self.client_context.set_data(path, value, version=version)
+        else:
+            self.client_context.set(path, value, version=version)
+
+    @connected
+    @ensure_params(Multi("paths"))
     def do_rm(self, params):
         """
         Remove the znode
 
-        rm <path>
+        rm <path> [path] [path] ... [path]
+
+        Examples:
+
+        > rm /foo
+        > rm /foo /bar
+
+        """
+        for path in params.paths:
+            try:
+                self.client_context.delete(path)
+            except NotEmptyError:
+                self.show_output("%s is not empty.", path)
+            except NoNodeError:
+                self.show_output("%s doesn't exist.", path)
+
+    def complete_rm(self, cmd_param_text, full_cmd, *rest):
+        completers = [self._complete_path for i in range(0, 10)]
+        return complete(completers, cmd_param_text, full_cmd, *rest)
+
+    @connected
+    @ensure_params(Required("path"), IntegerRequired("version"))
+    def do_check(self, params):
+        """
+        Checks that a path is at a given version (only works within a transaction)
+
+        check <path> <version>
 
         Example:
 
-        > rm /foo
+        > txn 'create /foo "start"' 'check /foo 0' 'set /foo "end"' 'rm /foo 1'
+
+        """
+        if not self.in_transaction:
+            return
+
+        self.client_context.check(params.path, params.version)
+
+    @connected
+    @ensure_params(Multi("cmds"))
+    def do_txn(self, params):
+        """
+        Create and execute a transaction
+
+        txn <cmd> [cmd] [cmd] ... [cmd]
+
+        Allowed cmds are check, create, rm and set. Check parameters are:
+
+        check <path> <version>
+
+        For create, rm and set see their help menu for their respective parameters.
+
+        Example:
+
+        > txn 'create /foo "start"' 'check /foo 0' 'set /foo "end"' 'rm /foo 1'
 
         """
         try:
-            self._zk.delete(params.path)
-        except NotEmptyError:
-            self.show_output("%s is not empty.", params.path)
+            with self.transaction():
+                for cmd in params.cmds:
+                    try:
+                        self.onecmd(cmd)
+                    except AttributeError:
+                        # silently swallow unrecognized commands
+                        pass
+        except BadVersionError:
+            self.show_output("Bad version.")
+        except NoNodeError:
+            self.show_output("Missing path.")
+        except NodeExistsError:
+            self.show_output("One of the paths exists.")
 
-    complete_rm = _complete_path
+    def transaction(self):
+        class TransactionInProgress(Exception): pass
+        class TransactionNotStarted(Exception): pass
+
+        class Transaction(object):
+            def __init__(self, shell):
+                self._shell = shell
+
+            def __enter__(self):
+                if self._shell._txn is not None:
+                    raise TransactionInProgress()
+
+                self._shell._txn = self._shell._zk.transaction()
+
+            def __exit__(self, type, value, traceback):
+                if self._shell._txn is None:
+                    raise TransactionNotStarted()
+
+                try:
+                    self._shell._txn.commit()
+                finally:
+                    self._shell._txn = None
+
+        return Transaction(self)
+
+    @property
+    def client_context(self):
+        """ checks if we are within a transaction or not """
+        return self._txn if self.in_transaction else self._zk
+
+    @property
+    def in_transaction(self):
+        """ are we inside a transaction? """
+        return self._txn is not None
+
+    def complete_txn(self, cmd_param_text, full_cmd, *rest):
+        completers = [self._complete_path for i in range(0, 10)]
+        return complete(completers, cmd_param_text, full_cmd, *rest)
 
     @connected
     @ensure_params()
@@ -1086,6 +1228,7 @@ class Shell(XCmd):
         """
         fmt_str = """state=%s
 sessionid=%s
+auth_info=%s
 protocol_version=%d
 xid=%d
 last_zxid=%d
@@ -1095,16 +1238,17 @@ server=%s
 data_watches=%s
 child_watches=%s"""
         self.show_output(fmt_str,
-                       self._zk.client_state,
-                       self._zk.sessionid,
-                       self._zk.protocol_version,
-                       self._zk.xid,
-                       self._zk.last_zxid,
-                       self._zk.session_timeout,
-                       self._zk.client,
-                       self._zk.server,
-                       ",".join(self._zk.data_watches),
-                       ",".join(self._zk.child_watches))
+                         self._zk.client_state,
+                         self._zk.sessionid,
+                         list(self._zk.auth_data),
+                         self._zk.protocol_version,
+                         self._zk.xid,
+                         self._zk.last_zxid,
+                         self._zk.session_timeout,
+                         self._zk.client,
+                         self._zk.server,
+                         ",".join(self._zk.data_watches),
+                         ",".join(self._zk.child_watches))
 
     @ensure_params(Optional("match"))
     def do_history(self, params):
@@ -1238,22 +1382,23 @@ child_watches=%s"""
             self.show_output(str(ex))
 
     @connected
-    @ensure_params(Required("path"))
-    @check_paths_exists("path")
+    @ensure_params(Multi("paths"))
     def do_rmr(self, params):
         """
         Delete a path and all its children
 
-        rmr <path>
+        rmr <path> [path] [path] ... [path]
 
-        Example:
+        Examples:
 
         > rmr /foo
+        > rmr /foo /bar
 
         """
-        self._zk.delete(params.path, recursive=True)
+        for path in params.paths:
+            self._zk.delete(path, recursive=True)
 
-    complete_rmr = _complete_path
+    complete_rmr = complete_rm
 
     @connected
     @ensure_params(Required("path"))
