@@ -245,3 +245,54 @@ class BasicCmdsTestCase(ShellTestCase):
 
         expected_output = u"%s/foo\n" % (self.tests_path)
         self.assertEqual(expected_output, self.output.getvalue())
+
+    def test_session_endpoint(self):
+        self.shell.onecmd("session_endpoint 0 localhost")
+        expected = u"No session info for 0.\n"
+        self.assertEqual(expected, self.output.getvalue())
+
+    def test_ephemeral_endpoint(self):
+        path = "%s/ephemeral" % (self.tests_path)
+        self.shell.onecmd("create %s 'foo' true" % (path))
+        self.shell.onecmd("ephemeral_endpoint %s localhost" % (path))
+        self.assertTrue(self.output.getvalue().startswith("0x"))
+
+    def test_transaction_simple(self):
+        """ simple transaction"""
+        path = "%s/foo" % (self.tests_path)
+        txn = "txn 'create %s x' 'set %s y' 'check %s 1'" % (path, path, path)
+        self.shell.onecmd(txn)
+        self.shell.onecmd("get %s" % (path))
+        self.assertEqual("y\n", self.output.getvalue())
+
+    def test_transaction_bad_version(self):
+        """ check version """
+        path = "%s/foo" % (self.tests_path)
+        txn = "txn 'create %s x' 'set %s y' 'check %s 100'" % (path, path, path)
+        self.shell.onecmd(txn)
+        self.shell.onecmd("exists %s" % (path))
+        self.assertIn("Path %s doesn't exist\n" % (path), self.output.getvalue())
+
+    def test_transaction_rm(self):
+        """ multiple rm commands """
+        self.shell.onecmd("create %s/a 'x' true" % (self.tests_path))
+        self.shell.onecmd("create %s/b 'x' true" % (self.tests_path))
+        self.shell.onecmd("create %s/c 'x' true" % (self.tests_path))
+        txn = "txn 'rm %s/a' 'rm %s/b' 'rm %s/c'" % (
+            self.tests_path, self.tests_path, self.tests_path)
+        self.shell.onecmd(txn)
+        self.shell.onecmd("exists %s" % (self.tests_path))
+        self.assertIn("numChildren=0", self.output.getvalue())
+
+    def test_zero(self):
+        """ test setting a znode to None (no bytes) """
+        path = "%s/foo" % (self.tests_path)
+        self.shell.onecmd("create %s bar" % path)
+        self.shell.onecmd("zero %s" % path)
+        self.shell.onecmd("get %s" % path)
+        self.assertEqual("None\n", self.output.getvalue())
+
+    def test_create_sequential_without_prefix(self):
+        self.shell.onecmd("create %s/ '' false true" % self.tests_path)
+        self.shell.onecmd("ls %s" % self.tests_path)
+        self.assertEqual("0000000000\n", self.output.getvalue())
