@@ -88,8 +88,8 @@ class JsonCmdsTestCase(ShellTestCase):
         self.shell.onecmd("create %s/b '%s'" % (self.tests_path, jsonstr))
         self.shell.onecmd("json_get %s a.b.c.d true" % (self.tests_path))
 
-        expected_output = "b: value\na: value\n"
-        self.assertEqual(expected_output, self.output.getvalue())
+        self.assertIn("a: value", self.output.getvalue())
+        self.assertIn("b: value", self.output.getvalue())
 
     def test_json_get_template(self):
         """ test get """
@@ -100,14 +100,14 @@ class JsonCmdsTestCase(ShellTestCase):
         self.assertEqual("key = value\n", self.output.getvalue())
 
     def test_json_get_recursive_template(self):
-        """ test get recursively """
+        """ test get recursively (template) """
         jsonstr = '{"a": {"b": {"c": {"d": "value"}}}}'
         self.shell.onecmd("create %s/a '%s'" % (self.tests_path, jsonstr))
         self.shell.onecmd("create %s/b '%s'" % (self.tests_path, jsonstr))
         self.shell.onecmd("json_get %s 'the value is: #{a.b.c.d}' true" % (self.tests_path))
 
-        expected_output = "b: the value is: value\na: the value is: value\n"
-        self.assertEqual(expected_output, self.output.getvalue())
+        self.assertIn("a: the value is: value", self.output.getvalue())
+        self.assertIn("b: the value is: value", self.output.getvalue())
 
     def test_json_count_values(self):
         """ test count values in JSON dicts """
